fbstHWE <-
function(X, delta=5e-3, simulate.p.value=FALSE, B=200, verbose=FALSE) {
   # simulate.p.value:	a logical indicating whether to compute p-values by Monte Carlo simulation.
   # B: an integer specifying the number of replicates used in the Monte Carlo test.

   #  Funcao para chamada do FBST para teste de equilibrio de hardy-weinberg 
   #  x eh um vetor representando as frequencias genotipicas (na forma de 
   #  uma matriz triangular inferior ou seja, x=c(n_11,n_21,n_31,...,n_k1,n_22,n_32,...)



#########
#
# Auxiliary functions
#

  mtcar = function(omega, resp, fdens="fdens", gsamp="gsamp", nmc=100, nst=1000, pack=1){
  
   # fdens(theta|X) - original pdf/likelihood
   # prototype:
   #    f = fdens(X,theta)
   #			X: data; theta: parameter vector
   #
   # gsamp(theta|X, omega) - importance sampling generating function
   # prototype:
   #    [theta, g, omega] = gsamp(X,omega,j,nst)
   #			X: data; omega: hyperparameters; nst: number of strata; j:current stratum; 
   #       theta: parameter vector; g: G(theta)
   #         1  2  3     4
   # resp= [xid,k,targ,chi2,
   #         5     6     7     8      9      10     11      12      13    14  
   #        f0[1],ah[1],aw[1],awh[1],awc[1],aw2[1],aw2h[1],aw2c[1],v[1],prec[1],
   #        f0[2],ah[2],aw[2],awh[2],awc[2],aw2[2],aw2h[2],aw2c[2],v[2],prec[2],...] 
  
  
   # Accumul, Mean, Std, Hit (f>f0), Complement, eVidence 
   # v= eVidence --against-- the hypothesis  
   # [mx,nx]= size(Xs);  
   # mx= Poissom pairs per X , 
   # nx= X samples in Xs for quantile computation   
   # X= Xs(1:mx,[2*xid-1,2*xid]);  
   # nmc= monte carlo points (t1,t2) 
   # nst= stratified points (t3) per (t1,t2) 
   # mx= Poissom pairs per X , 
   # nx= X samples in Xs for quantile computation   
  
   # Fieller and Finney confidence interval:  
   # sv= Does not exist! N(0,1)/N(0,1) is a Cauchy! 
   # v= mwh/mw = mwh/(mwh+mwc) , wh*wc=0 ,   
   # Cov(A*X) = A*Cov(X)*A' , Cov(X,Y) = E(X*Y)-E(X)*E(Y)    
   # Cov(wh,wc) = E(wh*wc) -E(wh)*E(wc) = -mwh*mwc 
   # Cov(wh,w) = Cov(wh,(wh+wc)) = swh^2  +Cov(wh,wc)    
   # sw^2 = Var(wh+wc) = swh^2 +swc^2   -2*mwh*mwc 
   # Cov(wh,w) = sw^2 -swc^2 +mwh*mwc 
   # v_n= mwh/mw  , u_n= wh -v*w , mu_n= mwh -v*mw 
   # E(u) = 0 , (1/n)*Var(wh -v*w) =  Var(mu) = 
   # =(1/n)*(sw1^2 -2*v*(sw^2-swc^2+mwh*mwc) +v^2*sw^2)  
   # n*(mu)^2/Var(mu)  ~ Chi2(r=1)  as n -> \infty  
   # => A*v^2 -2*B*v +C = 0 
   # A= mw^2 -sw^2*chi2/n  
   # B= mw*mwh -chi2*(sw^2 -swc^2 +mwh*mwc)/n  
   # C= mwh^2 -swh^2*chi2/n  
   # v= (2*B +-sqrt(4*B^2 -4*A*C))/(2*A) 
   # So, a confidence interval: v +-delta  
   # delta= sqrt(B^2 -A*C)/A  ~= sqrt(chi2*D/n) 
   # where D= rwh^2*(1-v)^2 +rwc^2*v^2 
   # on the coeficients (of vaRiation)     
   # rw= sw/mw , rwh= swh/mw , rwc= swc/mw   
   # Solving for n, 
   # n= chi2*D/d^2 , chi2= Chi2(df=1, 1- alpha)    
   # define chi2 (1-alpha)-quantile of Chi2(d.f.=1)
   # alpha     0.005  0.01   0.025  0.05  0.1  
   # (1-a)-q   7.88   6.63   5.02   3.84  2.71 
   
   EPS = 1e-5
   
   szf0 <- (length(resp)-4)/10
   if (szf0 %% 1 != 0)
    stop ("Erro em MTCAR: tamanho invalido do vetor de resultados")
  
   xid=  resp[1]
   k=    resp[2]
   targ= resp[3] 
   chi2= resp[4]
  
   f0 <- rep(0,szf0)
   ah <- rep(0,szf0)
   aw <- rep(0,szf0)
   awh <- rep(0,szf0)
   awc <- rep(0,szf0)
   aw2 <- rep(0,szf0)
   aw2h <- rep(0,szf0)
   aw2c <- rep(0,szf0)
   v <- rep(0,szf0)
   delta <- rep(0,szf0)
   
   #         1  2  3     4
   # resp= [xid,k,targ,chi2,
   #         5     6     7     8      9      10     11      12      13    14  
   #        f0[1],ah[1],aw[1],awh[1],awc[1],aw2[1],aw2h[1],aw2c[1],v[1],delta[1],
   #        f0[2],ah[2],aw[2],awh[2],awc[2],aw2[2],aw2h[2],aw2c[2],v[2],delta[2],...] 
  
   for (j in 1:szf0) {
      idx <- (j-1)*10+5
      f0[j]=   resp[idx]   
      ah[j]=   resp[idx+1]   
      aw[j]=   resp[idx+2]
      awh[j]=  resp[idx+3]
      awc[j]=  resp[idx+4]
      aw2[j]=  resp[idx+5]
      aw2h[j]= resp[idx+6]
      aw2c[j]= resp[idx+7] 
      v[j]= resp[idx+8]     
      delta[j]= resp[idx+9]
   }
  
   if(chi2==0) { chi2= 5.02 } # 6.7
   if(k==0) { mink = 5 } else { mink=0 }   
  
   # Importance sampling: g a prob density:  
   # Sh(y|x)dF = Sh(y|x)f(x)dx = 
   # Sh(y|x)(f(x)/g(x))g(x)dx = Sh(y|x)(f/g)dG 
   # MC: draw X_j ~g(x)  w_j= f(x_j)/g(x_j) 
   # S ~= sum{j=1:n}(w_j*h(y|x))/sum{j=1:n}(w_j) 
   
   #posterior/likelihood = import. sampl. parameters  
   
   #pack= 500 
   flag= 1
   wjant = 0
  
   while( flag==1 ) { 
     k= k+1  
     
     #for (j in 1:nst) {    #nst strat. samp. for theta
        
     sampling = eval(call(gsamp, nst, omega))
     theta = sampling$theta
     gj = sampling$dens
     omega = sampling$omega
  
     fj = eval(call(fdens, theta, omega))
     #print(paste(formatC(fj), " ", formatC(gj)))
     # Assumimos que fj e gj sao os logs das densidades
     wj=  exp(fj-gj) # se g e f sao a mesma funcao, wj = 1
  
     wj2= wj*wj
  
     for (i in 1:szf0) {
       hj <- rep(0,nst)
       hj[which(fj>=f0[i])] <- 1
       cj <- 1-hj
       
       ah[i] = ah[i]  + sum(hj) 
       aw[i] = aw[i]  + sum(wj)
       awh[i]=  awh[i] + sum(hj*wj)
       awc[i]=  awc[i] + sum(cj*wj) 
       aw2[i]=  aw2[i] + sum(wj2)
       aw2h[i]= aw2h[i] + sum(hj*wj2)
       aw2c[i]= aw2c[i] + sum(cj*wj2) 
     }
     #} #for (j in 1:nst)
     if( k %% pack ==0 ) {    
       n=    k*nst
       sqrtn= sqrt(n)
       
       for (i in 1:szf0) {
          mh=   ah[i]/n
          mw=   aw[i]/n
          mwh=  awh[i]/n
          mwc=  awc[i]/n
          mw2=  aw2[i]/n 
          mw2h= aw2h[i]/n  
          mw2c= aw2c[i]/n 
  
          #print(c(mh,mw,mwh,mwc,mw2,mw2h,mw2c))
  
          sh=   sqrt(max(mh  -mh^2,0)) 
          sw=   sqrt(max(mw2 -mw^2,0)) 
          swh=  sqrt(max(mw2h -mwh^2,0))  
          swc=  sqrt(max(mw2c -mwc^2,0))  
  
          vh=   mh
          v[i]=  mwh/mw   # evidencia contra H
          rw=   sw/mw  
          rwh= swh/mw
          rwc= swc/mw    
          A= mw^2 -sw^2*chi2/n   
          B= mw*mwh -chi2*(sw^2 -swc^2 +mwh*mwc)/n  
          C= mwh^2 -swh^2*chi2/n   
          delta[i] = sqrt(max(B^2 -A*C,0))/A 
          D= rwh^2*(1-v[i])^2 +rwc^2*v[i]^2  
          delta2= sqrt(chi2*D/n)
       }
       
       limv = cbind(v,1-v)*0.2
       matrpr = cbind(rep(targ,szf0), apply(limv,1,min))
       newtarg = apply(matrpr,1,min) 
       newtarg[which(newtarg==0)] = targ    
  
       idxdelta <- intersect(which(delta>=0),which(delta<newtarg))
       #print(c(szf0, idxdelta))
       if( (k >= mink) && (k >= nmc || length(idxdelta)==szf0))  
         flag= 0; 

       #print(c(xid, k, v, delta, min(newtarg)))
  
     }#if_mod_100 
   }#while 
  
   resp = c(xid,k,targ,chi2)
  
   for (j in 1:szf0) 
      resp <- c(resp, f0[j], ah[j], aw[j], awh[j], awc[j], aw2[j],
                aw2h[j], aw2c[j], v[j], delta[j])
   return(resp) 
  }           
  
  
  
  
  ##################
  #
  #  Gerador da distribuicaoo Dirichlet pelo metodo de transformacoes 
  #
  #  Se Z1, Z2 ... Zk possuem distribuicoes Gama com parametro de forma
  #   a1, a2 ... ak e um unico parametro de escala constante, entao  
  #   (Z1/S, Z2/S ... Sk/S), onde S = Z1+Z2+...+Zk, tem distribuicao
  #   Dirichlet com vetor de parametros (a1, a2 ... ak)
  #
  #  Versao adaptada da funcao rdirichlet do pacote MCMCpack
  #  A funcao original nao garantia que a soma de cada vetor aleatorio 
  #  gerado fosse igual a 1 (o que  obrigatorio!)
  #  A funcao abaixo garante esta condicao.
  #
  rdirichlet2 = function (n, alpha)
  {
      l <- length(alpha)
      p = matrix(0, nrow=n, ncol=l)  # prepara matriz de saida
      out = 1:n                      # posicoes de p que precisam ser geradas
  
      while (length(out)>0) {  # enquanto houver vetores com soma <> 1
  
        # as proximas 3 linhas geram vetores com distribuicao dirichlet
  
        # gera gamas com parametros de forma alpha[1], alpha[2], etc
        x <- matrix(rgamma(l*length(out), alpha), ncol = l, byrow = TRUE) 
        sm <- apply(x,1,sum)
        
        # ( x[i,] / sum(x[i,]) ) tem distribuicao Dirichlet com parametros 
        #  (alpha[1], alpha[2] ... alpha[k])
        prop = x / as.vector(sm, mode="numeric")
  
        # normalizacao de escala para aproximar a soma 1
        p[out,] = prop/apply(prop,1,sum)
        
        # verifica se hah vetores com soma <> 1
        out = which(1-apply(p,1,sum)!=0)
        #print(length(out))
      }
  
      return(p)
  }
  
  #
  #  Function for computing the number of alleles in a genotype vector 
  #  of length s
  #
  qtAlleles = function(s) {
     if (s>3) {
       qtalleles = suppressWarnings(as.integer(polyroot(c(-2*s, 1, 1))[1]))
     } else { qtalleles = 2 }
  
     return(qtalleles)
  }
  
  
  #
  # Funcao para calculo da densidade a posteriori
  # theta: matriz onde cada linha corresponde a um ponto do esp. parametrico
  # omega: vetor hiperparametros da dirichlet (mesma dimensao da amostra original)
  #
  lfxhw = function(theta, Omega) {
    if (is.vector(theta)) {
      MatrTheta = matrix(theta, nrow=1)   
    } else {
      MatrTheta = theta
    }
    szom = length(Omega)
    
    if (ncol(MatrTheta) != szom) {
      stop('Dimensoes incompativeis')
    }
    
  	lf = as.vector(log(MatrTheta)%*%(Omega-1))
  
  	return(lf)
  }
  
  
  #
  # Funcao para amostragem de pontos
  #
  gsamphw = function(n, omega) {
  
    MatrTheta = rdirichlet2(n, omega)
    g = lfxhw(MatrTheta, omega)
  
    gs <- list(theta = MatrTheta, dens = g, omega = omega)
  	return(gs)
  }
  
  #
  #  Funcao para calculo da evidencia normalizada contra a hipotese no espaco amostral
  #  ("p-valor")
  fbst.cumevidag = function(evidag, dimens, dimensh) {
    p = pchisq(qchisq(evidag, dimens),dimens-dimensh)
    return (p)
  }
  
  #
  #  Funcao para calculo do nivel critico de evidencia contra a hipotese
  #  dado um nivel de significancia
  #  Entrada: vetor alfa contendo os niveis de significancia desejados
  #  Saida: Vetor cl tal que, se evidag > cl[i], entao rejeita-se a hipotese com
  #         significancia alfa[i] 
  #
  fbst.clcumevidag = function(alfa, dimens, dimensh) {
    #cl = qchisq(1-alfa,dimens-dimensh)
    cl = pchisq(qchisq(1-alfa,dimens-dimensh), dimens)
    return (cl)
  }
  
  
  #
  #  Function for converting 
  #
  #   Supondo que uma matriz triangular inferior A de ordem k seja armazenada em um vetor
  #    [A11, A21, A22 ... Ak1, Ak2 ... Akk],
  #   esta funcao devolve o indice no vetor correspondente  posicao (i,j) da
  #   matriz original.
  posvect = function(i, j, k) {
    return((i*(i-1))/2 + j)
  }
  
  
  #
  # This function finds the point of maximum likelihood under the hypothesis
  #
  maxhyp = function(x) {
  
    szx = length(x)
    qtalleles = qtAlleles(szx) 
  
    afreq = rep(0,qtalleles)  # sum of allele frequencies
    for (i in 1:qtalleles) {
      afreq[i] = sum(x[posvect(i, 1:i, qtalleles)]) + 
                 sum(x[posvect(i:qtalleles, i, qtalleles)])
    }
    
    p0 = afreq / sum(afreq)
    p0[qtalleles] = 1-sum(p0[-qtalleles])
    th0 = rep(0, szx)
    for (i in 1:qtalleles) {
      for (j in 1:i) {
        if (i==j) {
          th0[posvect(i,i,qtalleles)] = p0[i]^2
        } else {
          th0[posvect(i,j,qtalleles)] = 2*p0[i]*p0[j]
        }
      }
    }
    th0[szx] = 1-sum(th0[-szx])
  
    return(list(p0=p0, th0=th0))
  }

  #
  #  Core function for computing the evidence in favor of the hypothesis
  #  x: vector representing a lower triangular matrix with genotype counts
  #  prec: see main function above
  #  nmcmax: see main function
  #
  fbstEvid = function(x, delta=0.005, nmc=0) {

    szx = length(x)
    qtalleles = qtAlleles(szx) 
    
    if (nmc==0) nmc = 300*szx;  
    
    dimens = szx - 1
    dimensh = qtalleles - 1
    
    omega = x+1.00001;   # priori uniforme
    
    # estimativa de max. posteriori irrestrita
    thetaop = (omega-1)/(sum(omega-1))
    thetaop[szx] = 1 - sum(thetaop[-szx])
    fop = lfxhw(thetaop,omega);
    
    # estimativa de max. posteriori sob hipotese de homogeneidade
    p0th0 = maxhyp(x)
    p0 = p0th0$p0
    thetaoph = p0th0$th0
    
    foph = lfxhw(thetaoph,omega);
    
    #         1  2  3     4
    # resp= [xid,k,targ,chi2,
    #         5     6     7     8      9      10     11      12      13    14  
    #        f0[1],ah[1],aw[1],awh[1],awc[1],aw2[1],aw2h[1],aw2c[1],v[1],delta[1],
    #        f0[2],ah[2],aw[2],awh[2],awc[2],aw2[2],aw2h[2],aw2c[2],v[2],delta[2],...] 
    
    resp = c(1,0,delta,0,foph,0,0,0,0,0,0,0,0,0)
    resp = mtcar(omega, resp, 'lfxhw', 'gsamphw', nst=1000, nmc=nmc)
    
    ev = 1-resp[13]       # evidencia a favor da hipotese
    prec = resp[14]
    
    return(list(ev=ev, prec=prec, fop=fop, foph=foph, thetaop=thetaop, thetaoph=thetaoph, 
                dimens=dimens, dimensh=dimensh))
  }


  #
  # Simula amostras para o teste de HWE
  #   counts: vetor com frequencias genotipicas n11, n21 ... nk1, n22, n32 ... nk2 ... nkk 
  #   n: tamanho de cada amostra gerada
  #   m: numero de amostras geradas
  #   hwe: amostras geradas com proporcoes sob HWE (true) ou proporcoes sorteadas via distr. 
  #        Dirichlet com parametros n11+1, n21+1, ...
  #
  simula.hw = function(counts, n=0, m=500, hwe=TRUE, verbose=FALSE) {
  
    szx = length(counts)
    qtalleles = qtAlleles(szx) 

    dimens = szx - 1
    dimensh = qtalleles - 1
    if (n==0) { n=sum(counts) }
  
    thetaoph = maxhyp(counts)$th0
  
    Samples = matrix(0,nrow=m, ncol=szx)
    evids = rep(0,m)
    
    for (i in 1:m) {
    	if (hwe==TRUE) {
    		theta=thetaoph
    	} else {
    		theta=rdirichlet2(1,counts+1)
    	}

      x = as.vector(rmultinom(1,n,theta))
  
      evid = fbstEvid(x)
  		
      Samples[i,]=x
      evids[i] = evid$ev
      if (verbose) {
        if ((i-1)%%10 == 0) {
          print(paste('Empirical p-value - sample', i, ', evid=', evids[i],
                      ', prec=', evid$prec))
        }
      }
      
    }
    return(list(Samples=Samples, evids=evids))
    
  }

########
# 
#  Inicio do Corpo principal da funcao
#

  if (is.matrix(X)) {
    if (nrow(X) != ncol(X)) {
      stop("Error: Invalid dimension of input data")
    }
    qtlin = nrow(X)
    szx = qtlin*(qtlin+1)/2
    counts = rep(0,szx)
    for (i in 1:qtlin) {
      for (j in 1:i) {
        if (i==j) {
          counts[posvect(i, j, qtlin)] = X[i,i]
        } else {
          counts[posvect(i, j, qtlin)] = X[i,j]+X[j,i]
        }
      }
    }
  } else {
    counts = X
  }
  
  fbstresults = fbstEvid(counts, delta)

  ev = fbstresults$ev
  
  if (simulate.p.value) {
    Samples = simula.hw(counts, m=B, verbose=verbose)
    evids = Samples$evid

    # return the empirical p.value  (North et al, 2002)
    emp.p.value = (length(which(evids<=ev)) + 1) / (B+1)
  } else {
    emp.p.value = NA
  }
  
  p.value = 1 - fbst.cumevidag(1-ev, fbstresults$dimens, fbstresults$dimensh)
  
  return(list(evid=ev, prec=fbstresults$prec, p.value=p.value, emp.p.value=emp.p.value, 
              fop=fbstresults$fop, foph=fbstresults$fop, 
              thetaop=fbstresults$thetaop, thetaoph=fbstresults$thetaoph, 
              dimens=fbstresults$dimens, dimensh=fbstresults$dimensh))
}
