\name{fbstHWE}
\alias{fbstHWE}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Full Bayesian Significance Test (FBST) for the Hardy-Weinberg Equilibrium
}
\description{
 Performs the test for Hardy-Weinberg Equilibrium (HWE) in a locus with
 several alleles A1, A2,..., Ak, k>=2,
 and returns the hypothesis evidence Ev(H).  
}
\usage{
fbstHWE(X, delta = 0.005, simulate.p.value = FALSE, B = 200, verbose = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{
    vector of genotype counts in a lower triangular form:
    \tabular{lll}{
    \tab X = c(
    \tab     x11, \cr
    \tab     x21, x22, \cr
    \tab     x31, x32, x33, \cr
    \tab     ... \cr
    \tab     xk1, xk2, ..., xkk \cr
    \tab where k is the number of alleles \cr \cr
    
    X may also contain a square matrix with genotype counts.
    }
}
  \item{delta}{ 
     precision parameter 0 < delta < 1, it corresponds to the maximum length 
     for the 95\% confidence interval ev(H)
}
  \item{simulate.p.value}{
    boolean parameter indicating whether to compute the 
	  Empirical p-value by Monte Carlo simulation. 
	  If FALSE, only asymptotic p-value is returned.
}
  \item{B}{
    number of samples to be drawn for the Empirical p-value computation
}
  \item{verbose}{
    verbose method. If TRUE, prints the estimated evidences and 
    respective precisions for simulated samples during the computation of 
    empirical p-value computation
}
}
\details{
  Given k alleles A1, A2, ..., Ak, k>=2,
  in a locus, and their genotype counts in lower triangular form 
  X = (x_ij), 1<=j<=i<=k, this function computes the
  evidence for the Hardy-Weinberg Equilibrium) HWE. 
 
  The parameter of interest, denoted by theta_ij, is the population relative 
  frequencies of genotypes AiAj (1<=j<=i<=k). Assuming a multinomial
  model for x and a Dirichlet(1,1,...,1) priori for theta, the posterior p.d.f
  is f(theta | x) = Dirichlet( x_11+1, x_12+1, ..., x_kk+1 ).

  Parameter space:
    THETA = {theta = (theta_ij), 1<=j<=i<=k | 0 <= thetax_ij <= 1, sum(theta)=1} 
   
    Parameter space under HWE: 
    H = {theta in THETA | there exists a vector p=(p_1, p_2,..., p_k), 
         0 <= p <= 1, sum(p)=1, such that:  
         theta_ij = (p_i)^2 if i=j; theta_ij = 2*p_i*p_j if i<>j}

	
    The evidence measure against the hypothesis is computed in two steps:
     1) compute f_0 = max_H f(theta|x)
	 	where f(theta|x) denotes the posterior probability density function.
	 
     2) compute the integral   
          evb = [ int_H f(theta|x) dtheta ] / [ int_THETA f(theta|x) dtheta ] 
          where T = {theta in THETA | f(theta | x) > f_0}

	The evidence measure suporting the hypothesis is:
                  evid = 1 - evb
                  
                  
}
\value{
  \item{evid}{evidence of HWE hypothesis}
  \item{prec}{95\% confidence interval length for evid}
  \item{p.value}{estimated asymptotic p-value}
  \item{emp.p.value}{empirical p-value (=NA if simulate.p.value=FALSE)}
}
\references{
1. M.S.Lauretto, F.Nakano, S.R.Faria Jr, C.A.B.Pereira, J.M.Stern. 
A straightforward multiallelic significance test for the Hardy-Weinberg
equilibrium law. Genetics and Molecular Biology 32(3), 619-625, 2009.

2.Pereira, C.A.B., Stern, J.M. Evidence and Credibility:
 Full Bayesian Significance Test for Precise Hypotheses.
 Entropy 1, 99-110, 1999.
            
3.C.A.B.Pereira, J.M.Stern, S. Wechsler. Can a significance test
be genuinely Bayesian? Bayesian Analysis 3(1), 79-100, 2008.
}
\author{
Marcelo S. Lauretto, Jaime A. Maranhao
}
\note{

}


\seealso{

}
\examples{
################################################
# Example 1:
# Extracted from Louis and Dempster (1987), 
# An exact test for Hardy-Weinberg and multiple alleles. 
# Biometrics 43:805-811
# 

X = c(
  0,			
  3,  1,		
  5, 18,  1,	
  3,  7,  5,  2)

EvFBST1 = fbstHWE(X, simulate.p.value=TRUE, B=100, verbose=TRUE)
print(c(EvFBST1$evid, EvFBST1$delta, EvFBST1$p.value, EvFBST1$emp.p.value))

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
